from multiprocessing import Process,  Queue

class node:
    def __init__(self, word):
        self._word = word
        self._neighbors = []
        self._status = 0
        self._discoveredby = 0

    def getWord(self):
        return self._word

    def getNeighbors(self):
        return self._neighbors

    def addNeighbor(self, neighbor_index):
        self._neighbors.append(neighbor_index)

    def isUnseen(self):
        if self._status == 0:
            return True
        else:
            return False

    def isSeen(self):
        if self._status == 1:
            return True
        else:
            return False

    def setUnseen(self):
        self._status = 0

    def setSeen(self):
        self._status = 1

    def discover(self, n):
        self._discoveredby = n

    def discovered(self):
        return self._discoveredby

def addWordLink(wordlist, word1, word2):
    for i in range(len(wordlist)):
        if wordlist[i].getWord() == word1:
            n1 = i
        if wordlist[i].getWord() == word2:
            n2 = i
    wordlist[n1].addNeighbor(n2)
    wordlist[n2].addNeighbor(n1)

def addLink(wordlist, index1, index2):
    wordlist[index1].addNeighbor(index2)
    wordlist[index2].addNeighbor(index1)

class queue:
    def __init__(self):
        self._queue = []

    def enqueue(self, x):
        self._queue.append(x)

    def dequeue(self):
        return self._queue.pop(0)

    def isEmpty(self):
        return len(self._queue) == 0

def retrievePath(nodelist, start, goal):
    # Return the path from start to goal
    if start == goal:
        path = []
        path.append(start)
        return path
    else:
        previous = nodelist[goal].discovered()
        previous_path = retrievePath(nodelist, start, previous)
        previous_path.append(goal)
        return previous_path

def BFS(nodelist, start, goal):
    to_visit = queue()
    nodelist[start].setSeen()
    to_visit.enqueue(start)

    found = False
    while (not found) and (not to_visit.isEmpty()):
        current = to_visit.dequeue()
        neighbors = nodelist[current].getNeighbors()
        for neighbor in neighbors:
            if nodelist[neighbor].isUnseen():
                nodelist[neighbor].setSeen()
                nodelist[neighbor].discover(current)
                if neighbor == goal:
                    found = True
                else:
                    to_visit.enqueue(neighbor)
    if found:
        return retrievePath(nodelist, start, goal)
    else:
        return []

def compareWords(word1, word2):
    if len(word1) != len(word2):
        return False
    numdifferent = 0
    for i in range(len(word1)):
        if word1[i] != word2[i]:
            numdifferent += 1
    if numdifferent == 1:
        return True
    else:
        return False

def findlinks(words, q, starti, endi):
    for i in range(starti, endi):
        for j in range(i+1,len(words)):
            #compare word i and word j
            if compareWords(words[i].getWord(), words[j].getWord()):
                q.put((i,j))  #addLink(words, i, j)

if __name__ == '__main__':
    #Read in the words from a dictionary
    words = []
    dict = open("dictionary4letter.txt", 'r')
    for line in dict:
        word = node(line.strip())
        words.append(word)

    q = Queue()
    process_list = []
    for i in range(0,len(words)-100,100):
        p = Process(target=findlinks, args=(words,q, i, i+100))
        process_list.append(p)
    p = Process(target=findlinks, args=(words, q, i+100, len(words)))
    process_list.append(p)
    for p in process_list:
        p.start()
    while not q.empty():
        i, j = q.get()
        addLink(words, i, j)
    for p in process_list:
        p.join()

    while not q.empty():
        i, j = q.get()
        addLink(words, i, j)


    #Get starting and ending word
    word1 = input("What is the starting word? ")
    word2 = input("What is the ending word? ")

    index1 = -1
    index2 = -1
    for i in range(len(words)):
        if words[i].getWord() == word1:
            index1 = i
        if words[i].getWord() == word2:
            index2 = i
    if index1 == -1:
        print(word1,"was not in the dictionary.  Exiting.")
        exit(0)
    if index2 == -1:
        print(word2,"was not in the dictionary.  Exiting")
        exit(0)

    #Find a chain of words
    path = BFS(words, index1, index2)

    #Report the chain
    if path == []:
        print("There was no chain between those words, in my dictionary.")
    else:
        for index in path:
            print(words[index].getWord())


